'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regionAlign = require('@zippytech/region-align');

var _regionAlign2 = _interopRequireDefault(_regionAlign);

var _assign = require('../assign');

var _assign2 = _interopRequireDefault(_assign);

var _getPositionOffsets = require('./getPositionOffsets');

var _getPositionOffsets2 = _interopRequireDefault(_getPositionOffsets);

var _positionsMap = require('./positionsMap');

var _positionsMap2 = _interopRequireDefault(_positionsMap);

var _getArrowPosition = require('./getArrowPosition');

var _getArrowPosition2 = _interopRequireDefault(_getArrowPosition);

var _getConstrainRegion = require('./getConstrainRegion');

var _getConstrainRegion2 = _interopRequireDefault(_getConstrainRegion);

var _isPositionBottom = require('./isPositionBottom');

var _isPositionBottom2 = _interopRequireDefault(_isPositionBottom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var posiblePositions = Object.keys(_positionsMap2.default);

// position
/**
 * Copyright (c) 2015-present, Zippy Technologies
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

function getPosition(_ref) {
  var _ref$constrainTo = _ref.constrainTo,
      constrainTo = _ref$constrainTo === undefined ? true : _ref$constrainTo,
      targetNode = _ref.targetNode,
      overlayNode = _ref.overlayNode,
      _ref$offset = _ref.offset,
      offset = _ref$offset === undefined ? 0 : _ref$offset,
      _ref$positions = _ref.positions,
      positions = _ref$positions === undefined ? posiblePositions : _ref$positions,
      _ref$relativeToViewpo = _ref.relativeToViewport,
      relativeToViewport = _ref$relativeToViewpo === undefined ? true : _ref$relativeToViewpo,
      _ref$arrowSize = _ref.arrowSize,
      arrowSize = _ref$arrowSize === undefined ? 10 : _ref$arrowSize,
      _ref$adjustOnPosition = _ref.adjustOnPositionBottom,
      adjustOnPositionBottom = _ref$adjustOnPosition === undefined ? true : _ref$adjustOnPosition,
      _ref$showArrow = _ref.showArrow,
      showArrow = _ref$showArrow === undefined ? true : _ref$showArrow;

  if (!constrainTo || !overlayNode || !targetNode) {
    return null;
  }

  var constrain = (0, _getConstrainRegion2.default)(constrainTo, targetNode);

  var overlayRegion = _regionAlign2.default.from(overlayNode);
  var alignRegion = _regionAlign2.default.from(targetNode);

  var newRegion = overlayRegion.clone();
  var positionsNormalized = positions.map(function (position) {
    var normalizedPosition = void 0;

    if (_positionsMap2.default[position]) {
      normalizedPosition = _positionsMap2.default[position].position;
    } else {
      normalizedPosition = position.position || position;
    }

    return normalizedPosition;
  });

  var positionsOffsets = (0, _getPositionOffsets2.default)(positions, offset);
  var positionsOffsetsClone = positionsOffsets.map(function (offset) {
    return (0, _assign2.default)({}, offset);
  });

  var succesfullPosition = newRegion.alignTo(alignRegion, positionsNormalized, {
    constrain: constrain,
    offset: positionsOffsetsClone
  });

  var constrainedWidth = newRegion.getWidth() != overlayRegion.getWidth();
  var constrainedHeight = newRegion.getHeight() != overlayRegion.getHeight();

  var arrowConfig = null;
  if (showArrow) {
    arrowConfig = (0, _getArrowPosition2.default)({
      arrowSize: arrowSize,
      overlayRegion: newRegion,
      targetRegion: alignRegion,
      position: succesfullPosition
    });
  }

  var position = {
    top: newRegion.top,
    left: newRegion.left
  };

  /**
   * If it uses absolute positon position must be corrected.
   * Substract top and left from offsetParent (first parent that has position).
   */
  if (!relativeToViewport && overlayNode.offsetParent) {
    var offsetParentRegion = _regionAlign2.default.from(overlayNode.offsetParent);
    position.left -= offsetParentRegion.left;
    position.top -= offsetParentRegion.top;
  }

  /**
   * If position is bottom, the overlay
   * should be positioned relative to bottom.
   */
  if ((0, _isPositionBottom2.default)(succesfullPosition) && adjustOnPositionBottom) {
    position.bottom = -(overlayRegion.height + (alignRegion.height - position.top));
    delete position.top;
  }

  return {
    alignRegion: alignRegion,
    constrainedHeight: constrainedHeight,
    constrainedWidth: constrainedWidth,
    constrained: constrainedHeight || constrainedWidth,
    positionRegion: newRegion,
    arrowConfig: arrowConfig,
    position: position,
    succesfullPosition: succesfullPosition
  };
}

exports.default = getPosition;